package com.vsa.gestorincidencias;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Base64;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.vsa.gestorincidencias.utilidades.Constantes;
import com.vsa.gestorincidencias.utilidades.JSONParser;
import com.vsa.gestorincidencias.utilidades.ObtenerDatos;

public class Login extends Activity implements OnClickListener {

	// Variables de los elementos de la pantalla
	private EditText cajaTextoUsuario, cajaTextoPassword;
	private Button botonLogin, botonRegistro;

	private ProgressDialog pDialog;
	
	// Parseo del JSON
	JSONParser jsonParser = new JSONParser();
	
	// Inicializacin de los array de JSON
	private JSONArray jsArrayProyectos = null;
	private JSONArray jsArrayAsignacionProyectos = null;

	// Crea un HashMap para guardar el listado de empresas que posteriormente se comparar con
	// con el usuario conectado para obtener su empresa asignada
	HashMap<String, String> listadoEmpresasIdNombre = new HashMap<String, String>();

	
	// Este procedimiento devuelve el id de empresa asignada al id usuario conectado pasado
	// como parmetro	
	public String obtenerIdEmpresaAsignadaUsuario(String idUsuarioConectado, String autorizacion) {

		String idEmpresa = "";
		
		// Obtiene el listado de todos los proyectos dados de alta
		JSONObject jsonProyectos = jsonParser.ejecutarSolicitudHTTP(Constantes.PROYECTOS_URL,
				"GET", null, autorizacion, false);
		try {
			String respuestaServidor = jsonProyectos.getString(Constantes.TAG_RESPUESTA);

			// Si la respuesta del servidor es correcta
			if (respuestaServidor.equals("200")){
				jsArrayProyectos = jsonProyectos.getJSONArray(Constantes.TAG_PROYECTOS);

				// Bucle que recorre todos los resultados del array de JSON obtenido
				for (int i = 0; i < jsArrayProyectos.length(); i++) {
					JSONObject jsonDatosProyectos = jsArrayProyectos.getJSONObject(i);					
					/* Obtenemos todas los id_solicitud de empresas con sus nombres
					* y los guardamos en un hashmap para consultarlo posteriormente
					*/
					listadoEmpresasIdNombre.put
					(jsonDatosProyectos.getString(Constantes.TAG_ID).toString(), 
							jsonDatosProyectos.getString(Constantes.TAG_NOMBRE).toString());				
				}

				final Iterator <Entry<String, String>> it = listadoEmpresasIdNombre.entrySet().iterator();
				Entry<String, String> entry= null;
				String idEmpresaBuscada ="";
				String nombreEmpresaBuscada ="";

				while (it.hasNext()) {
					entry= it.next();
					idEmpresaBuscada = entry.getKey(); //clave
					nombreEmpresaBuscada = entry.getValue(); //valor
					
					/* Se consulta si es la empresa asignada del listado "listadoEmpresasIdNombre"
					 * y se devuelve la que corresponde
					 */
					if (empresaAsignadaUsuarioProyecto(idUsuarioConectado, idEmpresaBuscada, nombreEmpresaBuscada, autorizacion)){
						return idEmpresaBuscada;
					}
				}
			} else {
				Toast.makeText(Login.this, "Se ha producido un error obteniendo el identificador de empresa asociada en: "
						+ Constantes.PROYECTOS_URL, Toast.LENGTH_LONG).show();
			}

		} catch (JSONException e) {
			e.printStackTrace();
		}
		return idEmpresa;
	}


	// Este procedimiento booleano devuelve si la empresa y el usuario conectado 
	//se encuentran relacionados (ambos son pasados como parmetros)
	public boolean empresaAsignadaUsuarioProyecto(String idUsuarioConectado,
			String idEmpresaBuscada, String nombreEmpresaBuscada, String autorizacion) {

		boolean empresaEncontrada = false;
		
		// Se cambia la url de asignaciones para pasarle como paramtro el id de empresa
		String USUARIO_ASIGNADOS_PROYECTO_URL_modificada = 
				Constantes.USUARIOS_ASIGNADOS_PROYECTO_URL.replaceAll("idProyecto", idEmpresaBuscada);

		// Se obtienen las asignaciones de usuarios a empresas por cada proyecto
		// "projects/idProyecto/memberships.json"
		JSONObject jsonAsignacionesProyectos = jsonParser.ejecutarSolicitudHTTP
				(USUARIO_ASIGNADOS_PROYECTO_URL_modificada,"GET", null, autorizacion, false);

		try {
			String respuestaServidor = jsonAsignacionesProyectos.getString(Constantes.TAG_RESPUESTA);

			if (respuestaServidor.equals("200")){
				jsArrayAsignacionProyectos = jsonAsignacionesProyectos.getJSONArray
						(Constantes.TAG_ASIGNACIONES);					

				// Bucle que recorre todos los resultados del array de JSON obtenido
				for (int i = 0; i < jsArrayAsignacionProyectos.length(); i++) {
					JSONObject jsonDatosAsignacionProyectos = jsArrayAsignacionProyectos.getJSONObject(i);

					JSONObject jsonDatosUsuarioCreador = jsonDatosAsignacionProyectos.getJSONObject(Constantes.TAG_USUARIO);		

					if (idUsuarioConectado.equals
							(jsonDatosUsuarioCreador.getString(Constantes.TAG_ID).toString())){

						SharedPreferences sp = PreferenceManager
								.getDefaultSharedPreferences(Login.this);
						Editor edit = sp.edit();						
						edit.putString("nombreEmpresa", nombreEmpresaBuscada);					
						edit.commit();
						empresaEncontrada = true;
					}	
				}
			}
			else {				
				Toast.makeText(Login.this, "Se ha producido un error obteniendo el identificador de empresa asociada en: "
						+ Constantes.PROYECTOS_URL, Toast.LENGTH_LONG).show();				
			}
		}
		catch (JSONException e) {
			e.printStackTrace();
		}
		return empresaEncontrada;
	}


	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.login);

		// Campos de entrada del texto
		cajaTextoUsuario = (EditText) findViewById(R.id.usuario);
		cajaTextoPassword = (EditText) findViewById(R.id.password);

		// Botones
		botonLogin = (Button) findViewById(R.id.login);
		botonRegistro = (Button) findViewById(R.id.registro);


		// Metodos Listener
		botonLogin.setOnClickListener(this);
		botonRegistro.setOnClickListener(this);

	}

	@Override
	public void onClick(final View v) {
		// Dependiendo si se pulsa Login o Registro ejecuta una actividad u otra
		switch (v.getId()) {
		case R.id.login:
			new IdentificandoUsuario().execute();
			break;
		case R.id.registro:
			Intent i = new Intent(this, RegistroUsuario.class);
			startActivity(i);
			break;

		default:
			break;
		}
	}


	class IdentificandoUsuario extends AsyncTask<String, String, String> {

		@Override
		protected void onPreExecute() {
			super.onPreExecute();
			pDialog = new ProgressDialog(Login.this);
			pDialog.setMessage("Accediendo al sistema...");
			pDialog.setIndeterminate(false);
			pDialog.setCancelable(true);
			pDialog.show();
		}

		@Override
		protected String doInBackground(String... args) {

			// Se guardan los datos de usuaro y contrasea en las variables correspondientes
			String usuario = cajaTextoUsuario.getText().toString();
			String password = cajaTextoPassword.getText().toString();

			// Se codifican en BASE64 para enviarlo en la llamada al servidor
			String autorizacion = "Basic " + Base64.encodeToString(
					(usuario + ":" + password).getBytes(),
					Base64.NO_WRAP);
			
			
			String apiKeyUsuario = new ObtenerDatos()
			.obtenerApiKeyUsuario(Login.this, autorizacion);
			
			String rutaFicheroManualUsuario = new ObtenerDatos()
			.obtenerRutaFicheroManualUsuario(Constantes.TIPO_PROYECTO_INTERNO, Login.this, autorizacion);
			
			// Se realiza una llamada a la url de obtener solicitudes ISSUES, para comprobar
			// si el usuario identificado tiene permisos (esta dado de alta)
			// en caso afirmativo el servidor devolver 200
			JSONObject jsonLogin = jsonParser.ejecutarSolicitudHTTP
					(Constantes.LEER_TIPOS_SOLICITUD_URL,"GET", null, autorizacion, false);
			try {

				String respuestaServidor = jsonLogin.getString(Constantes.TAG_RESPUESTA);

				// Si el usuario se identifica corerectamente
				if (respuestaServidor.equals("200")){

					Log.d("Acceso correcto!", "prueba");

					// Se obtiene el id del usuario conectado llamando a la clase ObtenerDatos()
					String idUsuarioConectado = new ObtenerDatos()
					.obtenerIdUsuario(Login.this, autorizacion);

					String idEmpresa = obtenerIdEmpresaAsignadaUsuario(idUsuarioConectado, autorizacion);
					
					// Guarda los datos del usuario identificado en Preferencias compartidas
					SharedPreferences sp = PreferenceManager
							.getDefaultSharedPreferences(Login.this);
					Editor edit = sp.edit();
					edit.putString("usuario", usuario);
					edit.putString("idUsuario", idUsuarioConectado);
					edit.putString("password", password);
					edit.putString("autorizacionBasica", autorizacion);
					edit.putString("apiKeyUsuario", apiKeyUsuario);
					edit.putString("rutaFicheroManualUsuario", rutaFicheroManualUsuario);
					
					
					
					//Si pertenece a alguna empresa es de tipo cliente/cliente responsable
					// si no es personal tcnico
					if (idEmpresa != ""){
						String nombreRol = new ObtenerDatos()
						.obtenerNombreRol(idUsuarioConectado, idEmpresa, Login.this, autorizacion);
						edit.putString("nombreRol", nombreRol);	
					}
					
					edit.putString("idEmpresa", idEmpresa);									

					edit.commit();

					// Llamamos a la actividad del MenuPrincipal
					Intent i = new Intent(Login.this, MenuPrincipal.class);
					finish();
					startActivity(i);				
				} else {		
					return "Acceso erroneo!";
				}		

			} catch (JSONException e) {

				e.printStackTrace();

			}
			return "Acceso correcto!";
		}


		protected void onPostExecute(String file_url) {			
			pDialog.dismiss();
			if (file_url != null) {
				Toast.makeText(Login.this, file_url, Toast.LENGTH_LONG).show();
			}
		}
	}
}