package com.vsa.gestorincidencias;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.StrictMode;
import android.preference.PreferenceManager;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;

import com.vsa.gestorincidencias.utilidades.Constantes;
import com.vsa.gestorincidencias.utilidades.JSONParser;

public class MenuPrincipal extends Activity implements OnClickListener {

	// Variables de los elementos de la pantalla	
	private Button botonMisSolicitudes, botonRegistrarIncidencia,
	botonNuevoDesarrollo, botonSugerencias, botonFaqs, botonManual;

	private ProgressDialog pDialog;

	JSONParser jsonParser = new JSONParser();



	@Override
	protected void onCreate(Bundle savedInstanceState) {		
		super.onCreate(savedInstanceState);
		setContentView(R.layout.menu_principal);

		// Para evitar problemas con varios hilos
		StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
		StrictMode.setThreadPolicy(policy);

		// Botones
		botonMisSolicitudes = (Button) findViewById(R.id.mis_solicitudes);
		botonRegistrarIncidencia = (Button) findViewById(R.id.registrar_incidencia);
		botonNuevoDesarrollo = (Button) findViewById(R.id.nuevo_desarrollo);
		botonSugerencias = (Button) findViewById(R.id.sugerencias);
		botonFaqs = (Button) findViewById(R.id.faqs);
		botonManual = (Button) findViewById(R.id.manual_usuario);		

		// Metodos Listener
		botonMisSolicitudes.setOnClickListener(this);
		botonRegistrarIncidencia.setOnClickListener(this);
		botonNuevoDesarrollo.setOnClickListener(this);
		botonSugerencias.setOnClickListener(this);
		botonFaqs.setOnClickListener(this);
		botonManual.setOnClickListener(this);
	}	


	@Override
	public void onClick(View v) {

		// Se obtiene el rol del usuario conectado para comprobar si tiene permisos
		SharedPreferences sp = PreferenceManager
				.getDefaultSharedPreferences(MenuPrincipal.this);
		String nombreRol = sp.getString("nombreRol", null);		
		String apiKeyUsuario = sp.getString("apiKeyUsuario", null);
		String rutaFicheroManualUsuario = sp.getString("rutaFicheroManualUsuario", null);

		// Dependiendo si se pulsa una opcin u otra ejecuta una actividad u otra
		switch (v.getId()) {

		case R.id.mis_solicitudes:
			Intent i = new Intent(this, MisSolicitudes.class);
			finish();
			startActivity(i);			
			break;
		case R.id.registrar_incidencia:
			Intent j = new Intent(this, RegistrarIncidencia.class);
			startActivity(j);		
			break;
		case R.id.nuevo_desarrollo:
			//Si el usuario es Ciente no tendra acceso a solicitar nuevos desarrollos
			if (!nombreRol.equals(Constantes.ROL_CLIENTE)) {			
				Intent k = new Intent(this, SolicitudDesarrollo.class);
				startActivity(k);
				break;			
			} else {
				Toast.makeText(MenuPrincipal.this, "Su usuario no dispone de permisos para la funcionalidad solicitada!",
						Toast.LENGTH_LONG).show();
				Intent in = new Intent(getApplicationContext(),
						MenuPrincipal.class);
				startActivity(in);
				break;
			}
		case R.id.sugerencias:
			Intent l = new Intent(this, EnviarSugerencias.class);
			startActivity(l);
			break;
		case R.id.faqs:
			Intent m = new Intent(this, PreguntasFrecuentes.class);
			startActivity(m);
			break;
		case R.id.manual_usuario:			
			Uri uri = Uri.parse(rutaFicheroManualUsuario + "?key=" + apiKeyUsuario);
			Intent o = new Intent(Intent.ACTION_VIEW, uri);
			startActivity(o);
			break;		
		}
	}


	class AttemptLogin extends AsyncTask<String, String, String> {

		@Override
		protected void onPreExecute() {
			super.onPreExecute();
			pDialog = new ProgressDialog(MenuPrincipal.this);
			pDialog.setMessage("Accediendo al sistema...");
			pDialog.setIndeterminate(false);
			pDialog.setCancelable(true);
			pDialog.show();
		}

		@Override
		protected String doInBackground(String... args) {
			return null;
		}

		protected void onPostExecute(String file_url) {
			pDialog.dismiss();
		}
	}
}
