package com.vsa.gestorincidencias;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;
import android.widget.Toast;

import com.vsa.gestorincidencias.utilidades.Constantes;
import com.vsa.gestorincidencias.utilidades.JSONParser;
import com.vsa.gestorincidencias.utilidades.Ordenar;

public class MisSolicitudes extends ListActivity {

	private ProgressDialog pDialog;	

	// Parseo del JSON
	JSONParser jsonParser = new JSONParser();

	// Array de datos de las solicitudes en formato JSON
	private JSONArray jsArraySolicitudes = null;
	private JSONArray jsArrayCamposPersonalizados = null;

	// Mapea mediante HashMap clave valor los elementos (solicitudes)
	private ArrayList<HashMap<String, String>> listadoSolicitudes;	

	// Caja de texto para filtrar
	EditText cajaTextoFiltradoSolicitud;
	
	// Botones
	private Button botonMenu;
	

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.mis_solicitudes);		
		
		// Si se pulsa el botn volver se regresa a la actividad anterior "MisSolicitudes"
		botonMenu = (Button) findViewById(R.id.boton_menu);
		botonMenu.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				Intent in = new Intent(getApplicationContext(),
						MenuPrincipal.class);
				startActivity(in);
			}
		});
		
	}	
	

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Se crean las opciones del menu desplegable
		menu.add(Menu.NONE, 1, Menu.NONE, "Menu principal")
		.setIcon(R.drawable.ic_view_list);
		//menu.add(Menu.NONE, 2, Menu.NONE, "Mis solicitudes")
		//.setIcon(R.drawable.ic_format_list_bulleted_type);
		menu.add(Menu.NONE, 3, Menu.NONE, "Registrar incidencia")
		.setIcon(R.drawable.ic_bug);
		menu.add(Menu.NONE, 4, Menu.NONE, "Nuevo desarrollo")
		.setIcon(R.drawable.ic_code_braces);
		menu.add(Menu.NONE, 5, Menu.NONE, "Enviar sugerencias")
		.setIcon(R.drawable.ic_email_variant);
		menu.add(Menu.NONE, 6, Menu.NONE, "Preguntas frecuentes")
		.setIcon(R.drawable.ic_comment_question_outline);
		menu.add(Menu.NONE, 7, Menu.NONE, "Manual de usuario")
		.setIcon(R.drawable.ic_book_open_page_variant);
		return true;
	}



	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		// Se obtiene el rol del usuario conectado para comprobar si tiene permisos
		SharedPreferences sp = PreferenceManager
				.getDefaultSharedPreferences(MisSolicitudes.this);
		String nombreRol = sp.getString("nombreRol", null);
		String apiKeyUsuario = sp.getString("apiKeyUsuario", null);
		String rutaFicheroManualUsuario = sp.getString("rutaFicheroManualUsuario", null);

		switch (item.getItemId()) {
		case 1:
			Intent i = new Intent(this, MenuPrincipal.class);
			finish();
			startActivity(i);
			return true;
			//case 2:
			//	Intent j = new Intent(this, MisSolicitudes.class);
			//	finish();
			//	startActivity(j);
			//	return true;
		case 3:
			Intent k = new Intent(this, RegistrarIncidencia.class);
			finish();
			startActivity(k);
			return true;
		case 4:
			if (!nombreRol.equals(Constantes.ROL_CLIENTE)) {			
				Intent l = new Intent(this, SolicitudDesarrollo.class);
				startActivity(l);							
			} else {
				Toast.makeText(MisSolicitudes.this, "Su usuario no dispone de permisos para la funcionalidad solicitada",
						Toast.LENGTH_LONG).show();
				Intent in = new Intent(getApplicationContext(),
						MisSolicitudes.class);
				startActivity(in);			
			}
			return true;
		case 5:
			Intent m = new Intent(this, EnviarSugerencias.class);
			finish();
			startActivity(m);
			return true;
		case 6:
			Intent n = new Intent(this, PreguntasFrecuentes.class);
			finish();
			startActivity(n);
			return true;
		case 7:
			Uri uri = Uri.parse(rutaFicheroManualUsuario + "?key=" + apiKeyUsuario);
			Intent o = new Intent(Intent.ACTION_VIEW, uri);
			startActivity(o);
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}


	@Override
	protected void onResume() {
		super.onResume();
		// Carga las solicitudes de manera Asincrona
		new CargarSolicitudes().execute();
	}

	public void ordenar(View v) {
		// Si devuelve true corresponde a ordenacion ascendente y false descendente
		Collections.sort(listadoSolicitudes, new Ordenar(v.getId() == R.id.boton_asc));
		actualizarListView();
	}


	public void actualizarDatosJSON() {

		Log.d("Solicitado!", "Comenzando");

		// Instancia el arraylist que contiene los datos en formato JSON
		listadoSolicitudes = new ArrayList<HashMap<String, String>>();

		SharedPreferences sp = PreferenceManager
				.getDefaultSharedPreferences(MisSolicitudes.this);
		String autorizacion = sp.getString("autorizacionBasica", null);

		String idUsuarioActual = sp.getString("idUsuario", null);	

		// Se realiza una llamada para consultar las solicitudes		
		JSONObject jsonSolicitudes = jsonParser.ejecutarSolicitudHTTP(Constantes.SOLICITUDES_URL,
				"GET", null, autorizacion, false);		

		try {
			String respuestaServidor = jsonSolicitudes.getString(Constantes.TAG_RESPUESTA);

			// Si la respuesta del servidor es correcta (200)
			if (respuestaServidor.equals("200")){

				jsArraySolicitudes = jsonSolicitudes.getJSONArray(Constantes.TAG_SOLICITUDES);

				// Bucle que recorre todos los resultados del array de JSON obtenido
				for (int i = 0; i < jsArraySolicitudes.length(); i++) {
					JSONObject jsonDatosSolicitud = jsArraySolicitudes.getJSONObject(i);					

					// ID y TITULO DEL ISSUE
					String idSolicitud = jsonDatosSolicitud.getString(Constantes.TAG_ID);				
					String tituloSolicitud = jsonDatosSolicitud.getString(Constantes.TAG_TITULO);					

					// ESTADO y NOMBRE DEL ISSUE
					JSONObject jsonDatosEstadoSolicitud = jsonDatosSolicitud.getJSONObject(Constantes.TAG_ESTADO);			
					String estadoSolicitud = jsonDatosEstadoSolicitud.getString(Constantes.TAG_NOMBRE);

					// TIPO DEL ISSUE
					JSONObject jsonDatosTipoSolicitud = jsonDatosSolicitud.getJSONObject(Constantes.TAG_TIPO_SOLICITUD);			
					String tipoSolicitud = jsonDatosTipoSolicitud.getString(Constantes.TAG_NOMBRE);

					// USUARIO CREADOR DEL ISSUE
					JSONObject jsonDatosCreadorSolicitud = jsonDatosSolicitud.getJSONObject(Constantes.TAG_AUTOR);			
					String idUsuario = jsonDatosCreadorSolicitud.getString(Constantes.TAG_ID);

					// DESCRIPCION DEL ISSUE 
					//como es un parametro opcional se comprueba que existe
					String descripcionSolicitud = "";
					if (jsonDatosSolicitud.has(Constantes.TAG_DESCRIPCION)) {
						descripcionSolicitud = jsonDatosSolicitud.getString(Constantes.TAG_DESCRIPCION);				
					}

					// CAMPOS PERSONALIZADOS DEL ISSUE 
					//(como son parametros opcionales se comprueba si existen)
					//el formato sera el siguiente:
					/*"custom_fields": [{
						"id": 2,
						"name": "Presupuesto",
						"value": "presupuestoSolicitud de prueba..."
					}]*/
					String presupuestoSolicitud = "";
					if (jsonDatosSolicitud.has(Constantes.TAG_CAMPOS_PERSONALIZADOS)) {						
						jsArrayCamposPersonalizados = jsonDatosSolicitud.getJSONArray(Constantes.TAG_CAMPOS_PERSONALIZADOS);

						for (int j = 0; j < jsArrayCamposPersonalizados.length(); j++) {
							JSONObject jsonDatosCamposPersonalizados = jsArrayCamposPersonalizados.getJSONObject(j);
							if (jsonDatosCamposPersonalizados.getString(Constantes.TAG_NOMBRE).equals(Constantes.TAG_PRESUPUESTO)){
								presupuestoSolicitud = jsonDatosCamposPersonalizados.getString(Constantes.TAG_VALOR);		
							}
						}
					}

					//  Si el usuario creador del ISSUE es el mismo que esta conectado, y es de tipo 
					// {"Incidencia Sistemas", "Incidencia Software", "Desarrollo"} (Si est incluido en este conjunto)
					//el ISSUE se introduce en la lista de HashMap, para ser visualizado posteriormente				
					
					
					if (idUsuarioActual.equals(idUsuario) && 
							(Arrays.asList(Constantes.CONJUNTO_TIPOS_SOLICITUDES).contains(tipoSolicitud))){

						// Crea un HashMap
						HashMap<String, String> listaDeSolicitudes = new HashMap<String, String>();

						listaDeSolicitudes.put(Constantes.TAG_ID, idSolicitud);
						listaDeSolicitudes.put(Constantes.TAG_TITULO, tituloSolicitud);
						listaDeSolicitudes.put(Constantes.TAG_ESTADO, estadoSolicitud);
						listaDeSolicitudes.put(Constantes.TAG_TIPO_SOLICITUD, tipoSolicitud);
						listaDeSolicitudes.put(Constantes.TAG_DESCRIPCION, descripcionSolicitud);	
						listaDeSolicitudes.put(Constantes.TAG_PRESUPUESTO, presupuestoSolicitud);

						// Aade La lista mapeada del HashMAp a un ArrayList que
						// contiene las solicitudes
						listadoSolicitudes.add(listaDeSolicitudes);
					}
				}
			} else {
				Toast.makeText(MisSolicitudes.this, "Se ha producido un error al obtener las solicitudes del usuario",
						Toast.LENGTH_LONG).show();
			}
		} catch (JSONException e) {
			e.printStackTrace();
		}
	}


	// Inserta los datos parseados en el ListView
	private void actualizarListView() {
		// Crea un adaptador que configura la colocacin de los datos a mostrar
		final SimpleAdapter adapter = new SimpleAdapter(this, listadoSolicitudes,
				R.layout.elemento_lista_solicitudes, new String[] {  Constantes.TAG_ID, Constantes.TAG_TITULO, 
				Constantes.TAG_TIPO_SOLICITUD, Constantes.TAG_ESTADO, Constantes.TAG_DESCRIPCION,
				Constantes.TAG_PRESUPUESTO}, 
				new int[] {R.id.id_solicitud, R.id.titulo_solicitud, R.id.tipo_solicitud, 
				R.id.estado_solicitud, R.id.descripcion_solicitud, R.id.presupuesto_solicitud});

		setListAdapter(adapter);

		ListView listViewSolicitudes = getListView();

		listViewSolicitudes.setTextFilterEnabled(true);
		
		// Evento de pulsado que se lanzan si unos de los elementos (Solicitudes) del listado es pulsado
		listViewSolicitudes.setOnItemClickListener(new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				
				String idSolicitudSeleccionado = ((TextView) view.findViewById(R.id.id_solicitud))
						.getText().toString();
				String tituloSolicitudSeleccionado = ((TextView) view.findViewById(R.id.titulo_solicitud))
						.getText().toString();
				String tipoSolicitudSeleccionado = ((TextView) view.findViewById(R.id.tipo_solicitud))
						.getText().toString();
				String estadoSolicitudSeleccionado = ((TextView) view.findViewById(R.id.estado_solicitud))
						.getText().toString();
				String descripcionSolicitudSeleccionado = ((TextView) view
						.findViewById(R.id.descripcion_solicitud)).getText().toString();
				String presupuestoSolicitudSeleccionado = ((TextView) view
						.findViewById(R.id.presupuesto_solicitud)).getText().toString();

				// Se arranca la actividad de Visualizar Solicitud pasndole 6 parmetros
				Intent in = new Intent(getApplicationContext(),
						VisualizarSolicitud.class);

				// Se envian los datos del elemento seleccionado a la actividad de VisualizarSolicitud
				in.putExtra("idSolicitud", idSolicitudSeleccionado);
				in.putExtra("tituloSolicitud", tituloSolicitudSeleccionado);	
				in.putExtra("tipoSolicitud", tipoSolicitudSeleccionado);
				in.putExtra("estadoSolicitud", estadoSolicitudSeleccionado);
				// Descripcin y presupuestoSolicitud no son visibles en el listado, son campos ocultos
				// que se envian a la siguiente actividad
				in.putExtra("descripcionSolicitud", descripcionSolicitudSeleccionado);	
				in.putExtra("presupuestoSolicitud", presupuestoSolicitudSeleccionado);

				startActivity(in);
			}
		});

		// Campo de texto para filtrar las solicitudes
		cajaTextoFiltradoSolicitud = (EditText) findViewById(R.id.filtrado);
		// Evento que se lanza al introducir texto en el campo de texto de filtrado
		cajaTextoFiltradoSolicitud.addTextChangedListener(new TextWatcher() {

			public void afterTextChanged(Editable s) {
			}

			public void beforeTextChanged(CharSequence s, int start, int count, int after) {
			}

			public void onTextChanged(CharSequence s, int start, int before, int count) {
				adapter.getFilter().filter(s.toString());
			}
		});		
	}


	public class CargarSolicitudes extends AsyncTask<Void, Void, Boolean> {

		@Override
		protected void onPreExecute() {
			super.onPreExecute();
			pDialog = new ProgressDialog(MisSolicitudes.this);
			pDialog.setMessage("Cargando solicitudes...");
			pDialog.setIndeterminate(false);
			pDialog.setCancelable(true);
			pDialog.show();
		}

		@Override
		protected Boolean doInBackground(Void... arg0) {
			actualizarDatosJSON();
			return null;
		}

		@Override
		protected void onPostExecute(Boolean result) {
			super.onPostExecute(result);
			pDialog.dismiss();
			actualizarListView();
		}
	}
}