package com.vsa.gestorincidencias;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.StrictMode;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import com.google.zxing.client.android.CaptureActivity;
import com.google.zxing.client.android.Intents;
import com.vsa.gestorincidencias.utilidades.Constantes;
import com.vsa.gestorincidencias.utilidades.JSONParser;

public class RegistrarIncidencia extends Activity{

	//Cajas de texto y botones
	private EditText cajaTextoTituloIncidencia, cajaTextoDescripcionIncidencia;
	private Button botonRegistrar, botonMenu;

	// Combo desplegable para el listado de tipos de incidencia
	private Spinner comboListadoTiposIncidencia;

	private ProgressDialog pDialog;

	// Parseo del JSON
	JSONParser jsonParser = new JSONParser();

	private JSONArray jsArrayTiposIncidencia = null;

	// Listado de tipos de solicitud para su consulta por clave
	Map<String,String> listadoTipoSolicitudID = new HashMap<String, String>();


	@Override
	protected void onCreate(Bundle savedInstanceState) {	
		super.onCreate(savedInstanceState);
		setContentView(R.layout.registrar_incidencia);

		cajaTextoTituloIncidencia = (EditText) findViewById(R.id.titulo_incidencia);
		cajaTextoDescripcionIncidencia = (EditText) findViewById(R.id.descripcion_incidencia);

		comboListadoTiposIncidencia = (Spinner) findViewById(R.id.listado_tipo_incidencias);

		StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
		StrictMode.setThreadPolicy(policy);

		// Llama al procedimient que aade los tipos de incidencia en el combo (Spinner)
		anadirElementosTipoIncidencia();

		// Boton registrar y su evento OnClick
		botonRegistrar = (Button) findViewById(R.id.registrar_incidencia);	
		botonRegistrar.setOnClickListener(new OnClickListener() {
			@Override
			// Si se pulsa el botn registrar (Solo hay un botn en esta pantalla)
			public void onClick(View v) {

				// Comprueba el resultado de la ejecucin
				String tituloIncidencia = cajaTextoTituloIncidencia.getText().toString();
				String descripcionIncidencia = cajaTextoDescripcionIncidencia.getText().toString();			
				String tipoIncidencia = comboListadoTiposIncidencia.getSelectedItem().toString();

				// Obtiene los datos del usuario almacenado en preferencias compartidas (usuario conectado)
				SharedPreferences sp = PreferenceManager
						.getDefaultSharedPreferences(RegistrarIncidencia.this);
				String idEmpresa = sp.getString("idEmpresa", null);
				String autorizacion = sp.getString("autorizacionBasica", null);

				try {

					JSONObject jsonEntradaIncidencia = new JSONObject();
					JSONObject jsonHijoEntradaIncidencia = new JSONObject();

					//Obtengo el id de la solicitud mediante el nombre de la solicitud
					// a traves del hashmap creado en la insercion del combo
					String idTipoIncidencia = listadoTipoSolicitudID.get(tipoIncidencia);

					// Construye los JSON de la Incidencia a registrar	
					jsonHijoEntradaIncidencia.put(Constantes.TAG_PROYECTO_ID, idEmpresa);				
					jsonHijoEntradaIncidencia.put(Constantes.TAG_TITULO, tituloIncidencia);
					jsonHijoEntradaIncidencia.put(Constantes.TAG_PRIORIDAD_ID, 4);
					jsonHijoEntradaIncidencia.put(Constantes.TAG_TIPO_SOLICITUD_ID, idTipoIncidencia);
					jsonHijoEntradaIncidencia.put(Constantes.TAG_DESCRIPCION, descripcionIncidencia);
					jsonEntradaIncidencia.put(Constantes.TAG_SOLICITUD, jsonHijoEntradaIncidencia);


					Log.d("Solicitud!", "Comenzando...");

					// Ejecuta el Post de la API REST para registrar la Incidencia				
					JSONObject jsonRegistroIncidencia =
							jsonParser.ejecutarSolicitudHTTP
							(Constantes.SOLICITUDES_URL, "POST", jsonEntradaIncidencia, autorizacion, false);

					String respuestaServidor = jsonRegistroIncidencia.getString(Constantes.TAG_RESPUESTA);

					// Si la respuesta es correcta
					if (respuestaServidor.equals("201")){				

						// Obtengo el id la solicitud registrada como respuesta del servidor tras su creacin
						JSONObject jsonDatosRespuestaIncidencia = jsonRegistroIncidencia.getJSONObject(Constantes.TAG_SOLICITUD);
						String idIncidencia = jsonDatosRespuestaIncidencia.getString(Constantes.TAG_ID);

						// Se guarda en preferencias compartidas el id de la ultima incidencia registrada
						// por el usuario para posteriormente consultarlo para actualizar con los datos 
						// del cdigo BIDI (Si es de sistemas)
						Editor edit = sp.edit();
						edit.putString("idUltimaIncidencia", idIncidencia);										
						edit.commit();

					} else {
						Log.d("Error al registrar incidencia!", 
								jsonRegistroIncidencia.getString(Constantes.TAG_ERRORES));
						Toast.makeText(RegistrarIncidencia.this, "Se ha producido un error al registrar la incidencia",
								Toast.LENGTH_LONG).show();
					}
				} catch (JSONException e) {
					e.printStackTrace();
				}				

				// Si la incidencia es de tipo Sistemas se llama a la actividad del Escaner de cdigos BIDI
				if (tipoIncidencia.equals(Constantes.TIPO_INCIDENCIA_SISTEMAS)){
					Intent intent = new Intent(RegistrarIncidencia.this, CaptureActivity.class);
					intent.putExtra("SCAN_MODE", "ONE_D_MODE");
					intent.putExtra("SCAN_FORMATS", "CODE_39,CODE_93,CODE_128,DATA_MATRIX,ITF,CODABAR,EAN_13,EAN_8,UPC_A,QR_CODE");
					intent.setAction(Intents.Scan.ACTION);
					startActivityForResult(intent, 1);

					new RegistrandoIncidencia().execute();

				} else {
					// Si la incidencia es de tipo Software, se finaliza la actividad
					Toast.makeText(RegistrarIncidencia.this, 
							"La incidencia de Software se ha registrado correctamente!",
							Toast.LENGTH_SHORT).show();
					finish();
				}
			}
		});


		// Si se pulsa el botn volver se regresa a la actividad anterior "MisSolicitudes"
		botonMenu = (Button) findViewById(R.id.boton_menu);
		botonMenu.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				Intent in = new Intent(getApplicationContext(),
						MenuPrincipal.class);
				startActivity(in);
			}
		});
	}


	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Se crean las opciones del menu desplegable
		menu.add(Menu.NONE, 1, Menu.NONE, "Menu principal")
		.setIcon(R.drawable.ic_view_list);
		menu.add(Menu.NONE, 2, Menu.NONE, "Mis solicitudes")
		.setIcon(R.drawable.ic_format_list_bulleted_type);
		//menu.add(Menu.NONE, 3, Menu.NONE, "Registrar incidencia")
		//.setIcon(R.drawable.ic_bug);
		menu.add(Menu.NONE, 4, Menu.NONE, "Nuevo desarrollo")
		.setIcon(R.drawable.ic_code_braces);
		menu.add(Menu.NONE, 5, Menu.NONE, "Enviar sugerencias")
		.setIcon(R.drawable.ic_email_variant);
		menu.add(Menu.NONE, 6, Menu.NONE, "Preguntas frecuentes")
		.setIcon(R.drawable.ic_comment_question_outline);
		menu.add(Menu.NONE, 7, Menu.NONE, "Manual de usuario")
		.setIcon(R.drawable.ic_book_open_page_variant);
		return true;
	}



	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		// Se obtiene el rol del usuario conectado para comprobar si tiene permisos
		SharedPreferences sp = PreferenceManager
				.getDefaultSharedPreferences(RegistrarIncidencia.this);
		String nombreRol = sp.getString("nombreRol", null);
		String apiKeyUsuario = sp.getString("apiKeyUsuario", null);
		String rutaFicheroManualUsuario = sp.getString("rutaFicheroManualUsuario", null);

		switch (item.getItemId()) {
		case 1:
			Intent i = new Intent(this, MenuPrincipal.class);
			finish();
			startActivity(i);
			return true;
		case 2:
			Intent j = new Intent(this, MisSolicitudes.class);
			finish();
			startActivity(j);
			return true;
			//case 3:
			//	Intent k = new Intent(this, RegistrarIncidencia.class);
			//	finish();
			//	startActivity(k);
			//	return true;
		case 4:
			if (!nombreRol.equals("Cliente")) {			
				Intent l = new Intent(this, SolicitudDesarrollo.class);
				startActivity(l);							
			} else {
				Toast.makeText(RegistrarIncidencia.this, "Su usuario no dispone de permisos para la funcionalidad solicitada",
						Toast.LENGTH_LONG).show();
				Intent in = new Intent(getApplicationContext(),
						RegistrarIncidencia.class);
				startActivity(in);			
			}
			return true;
		case 5:
			Intent m = new Intent(this, EnviarSugerencias.class);
			finish();
			startActivity(m);
			return true;
		case 6:
			Intent n = new Intent(this, PreguntasFrecuentes.class);
			finish();
			startActivity(n);
			return true;
		case 7:
			Uri uri = Uri.parse(rutaFicheroManualUsuario + "?key=" + apiKeyUsuario);
			Intent o = new Intent(Intent.ACTION_VIEW, uri);
			startActivity(o);
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}



	// Este procedimiento ser llamado tras leer el cdigo BIDI con el Escaner
	public void onActivityResult(int requestCode, int resultCode, Intent intent) {
		// Si la respuesta es correcta
		if (requestCode == 1 && resultCode == RESULT_OK) {
			final String datosCodigoBIDI = intent.getStringExtra(Intents.Scan.RESULT);

			SharedPreferences sp = PreferenceManager
					.getDefaultSharedPreferences(RegistrarIncidencia.this);
			String idIncidencia = sp.getString("idUltimaIncidencia", null);
			String autorizacion = sp.getString("autorizacionBasica", null);

			try {		

				// Construimos el JSON para insertar los datos del cdigo BIDI en el campo personalizado
				// Datos Incidencia
				JSONObject jsonEntradaDatosBIDI = new JSONObject();
				JSONObject jsonHijoDatosBIDI = new JSONObject();			
				JSONObject jsonCamposPersonalizados = new JSONObject();
				JSONArray jsonArrayCamposPersonalizados = new JSONArray();


				// Para obtener el id de un campo personalizado necesitamos permisos de administrador
				// lo pondremos directamente con el nmero id 1 que corresponde a "DatosIncidencia"
				jsonCamposPersonalizados.put(Constantes.TAG_ID, 1);
				jsonCamposPersonalizados.put(Constantes.TAG_NOMBRE, Constantes.TAG_DATOS_INCIDENCIA);
				jsonCamposPersonalizados.put(Constantes.TAG_VALOR, datosCodigoBIDI);

				jsonArrayCamposPersonalizados.put(jsonCamposPersonalizados);
				jsonHijoDatosBIDI.put(Constantes.TAG_CAMPOS_PERSONALIZADOS, jsonArrayCamposPersonalizados);
				jsonEntradaDatosBIDI.put(Constantes.TAG_SOLICITUD, jsonHijoDatosBIDI);

				// Obtenemos la URL exacta sobre la que lanza la actualizacin de la solicitud
				// "issues/idSolicitud.json"
				String SOLICITUD_URL_modificada = 
						Constantes.SOLICITUD_URL.replaceAll("idSolicitud", idIncidencia);

				// Ejecuta el PUT de la API REST para aadir el campo personalizado DatosIncidencia				
				JSONObject jsonActualizacinIncidencia =
						jsonParser.ejecutarSolicitudHTTP
						(SOLICITUD_URL_modificada, "PUT", jsonEntradaDatosBIDI, autorizacion, false);

				String respuestaServidor = jsonActualizacinIncidencia.getString(Constantes.TAG_RESPUESTA);

				// Si se actualiza correctamente finalizamos la actividad
				if (respuestaServidor.equals("200")){
					Log.d("Cdigo BIDI registrado correctamente!",
							jsonActualizacinIncidencia.toString());
					Toast.makeText(RegistrarIncidencia.this, 
							"La incidencia de Sistemas se ha registrado correctamente!",
							Toast.LENGTH_SHORT).show();
					finish();

				} else {
					Log.d("Error al registrar los datos de la incidencia mediante el cdigo BIDI!", 
							jsonActualizacinIncidencia.getString(Constantes.TAG_ERRORES));
					Toast.makeText(RegistrarIncidencia.this, 
							"Error al registrar los datos de la incidencia mediante el cdigo BIDI!",
							Toast.LENGTH_SHORT).show();
				}
			}catch (JSONException e) {
				e.printStackTrace();
			}				
		}else if (resultCode == RESULT_CANCELED) {
			Log.d("Error al leer el cdigo BIDI!", 
					"error");	
			Toast.makeText(RegistrarIncidencia.this, 
					"Error al leer el cdigo BIDI!",
					Toast.LENGTH_SHORT).show();
		}
	}


	// Se aaden los tipos de incidencia en el listado de tipos de incidencia
	public void anadirElementosTipoIncidencia() {

		// Obtiene el usuario almacenado en preferencias compartidas (usuario conectado)
		SharedPreferences sp = PreferenceManager
				.getDefaultSharedPreferences(RegistrarIncidencia.this);
		String autorizacion = sp.getString("autorizacionBasica", null);

		// Se realiza una llamada para obtener los tipos de incidencia
		JSONObject jsonTiposIncidencia = jsonParser.ejecutarSolicitudHTTP
				(Constantes.LEER_TIPOS_SOLICITUD_URL, "GET", null, autorizacion, false);
		try {			
			String respuestaServidor = jsonTiposIncidencia.getString(Constantes.TAG_RESPUESTA);

			// Si la respuesta el correcta
			if (respuestaServidor.equals("200")){
				jsArrayTiposIncidencia = jsonTiposIncidencia.getJSONArray(Constantes.TAG_TIPOS_SOLICITUD);

				comboListadoTiposIncidencia = (Spinner) findViewById(R.id.listado_tipo_incidencias);
				List<String> arrayListNombresTipoIncidencia = new ArrayList<String>();

				// Bucle que recorre todos los resultados del array de JSON obtenido
				for (int i = 0; i < jsArrayTiposIncidencia.length(); i++) {
					JSONObject jsonDatosTipoIncidencia = jsArrayTiposIncidencia.getJSONObject(i);

					// Solo insertamos las solicitudes que sean de tipo incidencia				
					if (jsonDatosTipoIncidencia.getString(Constantes.TAG_NOMBRE).
							toString().equals(Constantes.TIPO_INCIDENCIA_SOFTWARE) || 
							(jsonDatosTipoIncidencia.getString(Constantes.TAG_NOMBRE).
									toString().equals(Constantes.TIPO_INCIDENCIA_SISTEMAS))){
						arrayListNombresTipoIncidencia.add(jsonDatosTipoIncidencia.getString(Constantes.TAG_NOMBRE).toString());
					}

					//Se guardan los datos de las empresas/proyectos con su id_solicitud en el listado
					// de tipos de solicitud
					listadoTipoSolicitudID.put(jsonDatosTipoIncidencia.getString(Constantes.TAG_NOMBRE).toString(), 
							jsonDatosTipoIncidencia.getString(Constantes.TAG_ID).toString());
				}
				// Inserta los datos parseados en la lista mediante
				// un adaptador que configura la colocacin de los datos a mostrar
				ArrayAdapter<String> dataAdapter = new ArrayAdapter<String>(this,
						android.R.layout.simple_spinner_item, arrayListNombresTipoIncidencia);
				dataAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
				comboListadoTiposIncidencia.setAdapter(dataAdapter);
			} else {
				Toast.makeText(RegistrarIncidencia.this, 
						"Se ha producido un error al obtener los tipos de incidencia", Toast.LENGTH_LONG).show();
			}
		} catch (JSONException e) {
			e.printStackTrace();
		}
	}	


	class RegistrandoIncidencia extends AsyncTask<String, String, String> {

		@Override
		protected void onPreExecute() {
			super.onPreExecute();
			pDialog = new ProgressDialog(RegistrarIncidencia.this);
			pDialog.setMessage("Registrando incidencia...");
			pDialog.setIndeterminate(false);
			pDialog.setCancelable(true);
			pDialog.show();
		}

		protected void onProgressUpdate(String... values) {
			if (values != null) {
				for (String value : values) {
					Toast.makeText(RegistrarIncidencia.this, value,
							Toast.LENGTH_SHORT).show();
				}
			}
		}


		@Override
		protected String doInBackground(String... args) {
			return null;
		}		


		protected void onPostExecute(String file_url) {			
			pDialog.dismiss();
			if (file_url != null) {
				Toast.makeText(RegistrarIncidencia.this, file_url, Toast.LENGTH_LONG).show();
			}			
		}		
	}

}