package com.vsa.gestorincidencias;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.StrictMode;
import android.text.InputType;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import com.vsa.gestorincidencias.utilidades.Constantes;
import com.vsa.gestorincidencias.utilidades.JSONParser;
import com.vsa.gestorincidencias.utilidades.ObtenerDatos;

public class RegistroUsuario extends Activity{

	// Cajas de texto
	private EditText cajaTextoLogin, cajaTextoPassword, cajaTextoEmail,
	cajaTextoNombreUsuario, cajaTextoApellidosUsuario;
	
	// Combo desplegable para el listado de empresas
	private Spinner comboListadoEmpresas;
	
	// Boton y checkBox para seleccionar si es usuario responsable o no
	private Button botonRegistro, botonVolver;
	private CheckBox checkUsuarioResponsable;

	private ProgressDialog pDialog;

	// Parseo del JSON
	JSONParser jsonParser = new JSONParser();

	private JSONArray jsArrayProyectos = null;

	// listado de empresas/proyectos con sus ids
	Map<String,String> listadoEmpresasIdNombre = new HashMap<String, String>();	

	private String claveApiKey = "";

	@Override
	protected void onCreate(Bundle savedInstanceState) {

		super.onCreate(savedInstanceState);
		setContentView(R.layout.registro_usuario);

		cajaTextoLogin = (EditText) findViewById(R.id.usuario);
		cajaTextoPassword = (EditText) findViewById(R.id.password);
		cajaTextoEmail = (EditText) findViewById(R.id.email);
		cajaTextoNombreUsuario = (EditText) findViewById(R.id.nombre);
		cajaTextoApellidosUsuario = (EditText) findViewById(R.id.apellidos);		
		comboListadoEmpresas = (Spinner) findViewById(R.id.listado_empresas);

		checkUsuarioResponsable = (CheckBox) findViewById(R.id.check_usuario_responsable);

		StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
		StrictMode.setThreadPolicy(policy);		

		botonRegistro = (Button) findViewById(R.id.registro);
		botonRegistro.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				new CrearUsuario().execute();
			}
		});
		
		
		// Si se pulsa el botn volver se regresa a la actividad anterior "MisSolicitudes"
		botonVolver = (Button) findViewById(R.id.boton_volver);
		botonVolver.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				Intent in = new Intent(getApplicationContext(),
						Login.class);
				startActivity(in);
			}
		});

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle("Por favor, introduzca la clave (ApiKey)");

		//Se establece el campo de entrada para la claveApiKey
		final EditText cajaTextoClaveApiKey = new EditText(this);

		//Especifica el tipo de entrada esperada 
		//(se establece la entrada como una contrasea y enmascara el texto)
		cajaTextoClaveApiKey.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD);
		builder.setView(cajaTextoClaveApiKey);

		//Establece los botones de OK y cancelar
		builder.setPositiveButton("OK", new DialogInterface.OnClickListener() { 
			@Override
			public void onClick(DialogInterface dialog, int which) {
				claveApiKey = cajaTextoClaveApiKey.getText().toString();

				if (!comprobarClaveCorrecta(claveApiKey)) {
					Toast.makeText(RegistroUsuario.this, "La clave introducida es incorrecta!",
							Toast.LENGTH_LONG).show();
					Intent in = new Intent(getApplicationContext(),
							Login.class);
					startActivity(in);
				} else {
					Toast.makeText(RegistroUsuario.this, "Clave correcta!",
							Toast.LENGTH_LONG).show();
					// Llama a la funcin que rellena el listado de empresas
					anadirEmpresasCombo(claveApiKey);
				}

			}
		});
		builder.setNegativeButton("Cancelar", new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
				Intent in = new Intent(getApplicationContext(),
						Login.class);
				startActivity(in);
			}
		});			
		builder.show();		
	}
	
	// Procedimiento que devuelve si la claveApiKey de acceso de la API pasada como parmetro
	// para funciones de administrador es correcta
	public boolean comprobarClaveCorrecta(String claveApiKey) {
		boolean comprobarClave = false;

		JSONObject jsonUsuariosPruebaClave = jsonParser.ejecutarSolicitudHTTP(Constantes.USUARIOS_URL,
				"GET", null, claveApiKey, true);
		try {
			String respuestaServidor = jsonUsuariosPruebaClave.getString(Constantes.TAG_RESPUESTA);

			if (respuestaServidor.equals("200")){
				comprobarClave = true;
			} 
		} catch (JSONException e) {
			e.printStackTrace();
		}
		return comprobarClave;
	}


	// Se aaden las empresas/proyectos en el listado de empresas
	public void anadirEmpresasCombo(String claveApiKey) {					

		JSONObject jsonProyectos = jsonParser.ejecutarSolicitudHTTP(Constantes.PROYECTOS_URL,
				"GET", null, claveApiKey, true);
		try {
			String respuestaServidor = jsonProyectos.getString(Constantes.TAG_RESPUESTA);

			if (respuestaServidor.equals("200")){
				jsArrayProyectos = jsonProyectos.getJSONArray(Constantes.TAG_PROYECTOS);


				comboListadoEmpresas = (Spinner) findViewById(R.id.listado_empresas);
				List<String> list = new ArrayList<String>();

				// Bucle que recorre todos los resultados del array de JSON obtenido
				for (int i = 0; i < jsArrayProyectos.length(); i++) {
					JSONObject jsonDatosProyecto = jsArrayProyectos.getJSONObject(i);

					// Se aaden solo los proyectos cuyo identificador es "empresa"
					if (jsonDatosProyecto.getString(Constantes.TAG_IDENTIFICADOR).equals(Constantes.TAG_IDENTIFICADOR_EMPRESA)){

						list.add(jsonDatosProyecto.getString(Constantes.TAG_NOMBRE).toString());

						//Se guardan los datos de las empresas/proyectos con su id_solicitud
						listadoEmpresasIdNombre.put(jsonDatosProyecto.getString(Constantes.TAG_NOMBRE).toString(), 
								jsonDatosProyecto.getString(Constantes.TAG_ID).toString());
					}
				}

				ArrayAdapter<String> arrayAdapterEmpresas = new ArrayAdapter<String>(this,
						android.R.layout.simple_spinner_item, list);
				arrayAdapterEmpresas.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
				comboListadoEmpresas.setAdapter(arrayAdapterEmpresas);				

			} else {
				Toast.makeText(RegistroUsuario.this, "No ha sido posible cargar el listado de empresas",
						Toast.LENGTH_LONG).show();
			}
		} catch (JSONException e) {
			e.printStackTrace();
		}
	}



	class CrearUsuario extends AsyncTask<String, String, String> {

		@Override
		// Mtodo que se ejecuta mientras se realiza el registro del usuario
		protected void onPreExecute() {
			super.onPreExecute();
			pDialog = new ProgressDialog(RegistroUsuario.this);
			pDialog.setMessage("Registrando usuario...");
			pDialog.setIndeterminate(false);
			pDialog.setCancelable(true);
			pDialog.show();
		}

		@Override
		// Mtodo que se ejecuta en segundo plano durante el registro
		protected String doInBackground(String... args) {			

			String login = cajaTextoLogin.getText().toString();
			String password = cajaTextoPassword.getText().toString();
			String email = cajaTextoEmail.getText().toString();
			String nombreUsuario = cajaTextoNombreUsuario.getText().toString();
			String apellidosUsuario = cajaTextoApellidosUsuario.getText().toString();
			String nombreEmpresa = comboListadoEmpresas.getSelectedItem().toString();

			try {
				// Construccin de parmetros
				JSONObject jsonEntradaUsuario = new JSONObject();
				JSONObject jsonHijoEntradaUsuario = new JSONObject();

				//VALIDAR LAS SALIDAS DEL RESPONSE
				jsonHijoEntradaUsuario.put(Constantes.TAG_LOGIN, login);	
				jsonHijoEntradaUsuario.put(Constantes.TAG_NOMBRE_USUARIO, nombreUsuario);
				jsonHijoEntradaUsuario.put(Constantes.TAG_APELLIDOS, apellidosUsuario);
				//El mail lo valida tiene que ser con formato correcto	
				jsonHijoEntradaUsuario.put(Constantes.TAG_MAIL, email);
				jsonHijoEntradaUsuario.put(Constantes.TAG_PASSWORD, password);									

				jsonEntradaUsuario.put(Constantes.TAG_USUARIO, jsonHijoEntradaUsuario);

				Log.d("Solicitud realizada!", "Comenzando");

				// Ejecuta el Post de la API REST para registrar el usuario
				JSONObject jsonRegistroUsuario = jsonParser.ejecutarSolicitudHTTP
						(Constantes.USUARIOS_URL,
								"POST", jsonEntradaUsuario, claveApiKey, true);

				String respuestaServidorCreacionUsuario = jsonRegistroUsuario.getString(Constantes.TAG_RESPUESTA);

				if (respuestaServidorCreacionUsuario.equals("201")){
					//Se obtiene de la respuesta de creacion de la solicitud
					// el id del usuario creador
					JSONObject jsonDatosUsuarioCreado = jsonRegistroUsuario.getJSONObject(Constantes.TAG_USUARIO);

					String idUsuarioCreado = jsonDatosUsuarioCreado.getString(Constantes.TAG_ID);

					//Obtengo el id de la solicitud de la empresa mediante un hashmap
					//creado en la insercion del combo
					String idEmpresa = listadoEmpresasIdNombre.get(nombreEmpresa);

					// Llamamos a la funcin que asigna usuario a proyecto
					asignarUsuarioProyecto(idEmpresa, idUsuarioCreado);

				} else {
					Log.d("Se ha producido un error al registrar el usuario!",
							jsonRegistroUsuario.getString(Constantes.TAG_ERRORES));
					return "Se ha producido un error al registrar el usuario!";
				}
			} catch (JSONException e) {
				e.printStackTrace();
			}
			return null;
		}


		public void asignarUsuarioProyecto(String idEmpresa, String idUsuarioCreado) {

			// Se modifica la url para asignar el usuario a un proyecto que se pasa en la misma llamada
			// "projects/idProyecto/memberships.json"
			String ASIGNAR_USUARIO_PROYECTO_URL_modificada = 
					Constantes.USUARIOS_ASIGNADOS_PROYECTO_URL.replaceAll("idProyecto", idEmpresa);				

			JSONObject jsonEntradaAsignacionUsuario = new JSONObject();
			JSONObject jsonHijoAsignacionUsuario = new JSONObject();			

			try {
				jsonHijoAsignacionUsuario.put(Constantes.TAG_USUARIO_ID, idUsuarioCreado);
			
				JSONArray arrayRoles = new JSONArray();

				//Si se selecciona usuario responsable, se da de alta como usuario "Cliente responsable"
				if (checkUsuarioResponsable.isChecked()){
					// Roles es un array, podra tener varios roles, pero los clientes solo tendran un Rol
					arrayRoles = arrayRoles.put(new ObtenerDatos()
					.obtenerIdRol("Cliente responsable", RegistroUsuario.this, claveApiKey));
					// Si no se selecciona usuario responsable, se da de alta como usuario "Cliente"
				} else {
					arrayRoles = arrayRoles.put(new ObtenerDatos()
					.obtenerIdRol("Cliente", RegistroUsuario.this, claveApiKey));
				}
				
				// Se crea el JSON para asignar el proyecto al usuario
				jsonHijoAsignacionUsuario.put(Constantes.TAG_ROL_ID, arrayRoles);					

				jsonEntradaAsignacionUsuario.put(Constantes.TAG_ASIGNACION, jsonHijoAsignacionUsuario);

				// Ejecuta el Post de la API REST para asignar el usuario a su empresa/proyecto			
				JSONObject jsonAsignacionProyectoUsuario = jsonParser.ejecutarSolicitudHTTP
						(ASIGNAR_USUARIO_PROYECTO_URL_modificada,
								"POST", jsonEntradaAsignacionUsuario, claveApiKey, true);

				String respuestaServidorAsignacionProyecto = jsonAsignacionProyectoUsuario.getString(Constantes.TAG_RESPUESTA);

				// Si se actualiza correctamente
				if (respuestaServidorAsignacionProyecto.equals("201")){
					Log.d("Registrando...", jsonAsignacionProyectoUsuario.toString());
					Log.d("Usuario creado con exito!", jsonAsignacionProyectoUsuario.toString());
					finish();
					Toast.makeText(RegistroUsuario.this, "Usuario creado con xito!",
							Toast.LENGTH_LONG).show();
				} else {
					Log.d("Se ha producido un error en la asignacin de proyecto al usuario!",
							jsonAsignacionProyectoUsuario.getString(Constantes.TAG_ERRORES));
					Toast.makeText(RegistroUsuario.this, "Se ha producido un error en la asignacin de proyecto al usuario!",
							Toast.LENGTH_LONG).show();
				}
			} catch (JSONException e) {
				e.printStackTrace();
			}			
		}


		protected void onPostExecute(String file_url) {
			pDialog.dismiss();
			if (file_url != null) {
				Toast.makeText(RegistroUsuario.this, file_url,
						Toast.LENGTH_LONG).show();
			}
		}
	}
}
