package com.vsa.gestorincidencias;

import java.util.HashMap;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.StrictMode;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.vsa.gestorincidencias.utilidades.Constantes;
import com.vsa.gestorincidencias.utilidades.JSONParser;
import com.vsa.gestorincidencias.utilidades.ObtenerDatos;

public class SolicitudDesarrollo extends Activity{

	// Variables de los elementos de la pantalla
	private EditText cajaTextoTituloSolicitud, cajaTextoDescripcionSolicitud;
	private Button botonRegistrar, botonMenu;

	private ProgressDialog pDialog;

	// Parseo del JSON
	JSONParser jsonParser = new JSONParser();

	Map<String,String> listadoTipoSolicitudID = new HashMap<String, String>();


	@Override
	protected void onCreate(Bundle savedInstanceState) {	
		super.onCreate(savedInstanceState);
		setContentView(R.layout.solicitud_desarrollo);

		// Campos de entrada del texto
		cajaTextoTituloSolicitud = (EditText) findViewById(R.id.titulo_solicitud_desarrollo);
		cajaTextoDescripcionSolicitud = (EditText) findViewById(R.id.descripcion_solicitud_desarrollo);

		// Para evitar problemas con varios hilos
		StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
		StrictMode.setThreadPolicy(policy);

		// Boton registrar y su metodo listener
		botonRegistrar = (Button) findViewById(R.id.boton_registrar_nuevo_desarrollo);
		botonRegistrar.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				new RegistrandoSolicitudDesarrollo().execute();
			}
		});


		// Si se pulsa el botn volver se regresa a la actividad anterior "MisSolicitudes"
		botonMenu = (Button) findViewById(R.id.boton_menu);
		botonMenu.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				Intent in = new Intent(getApplicationContext(),
						MenuPrincipal.class);
				startActivity(in);
			}
		});

	}


	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Se crean las opciones del menu desplegable
		menu.add(Menu.NONE, 1, Menu.NONE, "Menu principal")
		.setIcon(R.drawable.ic_view_list);
		menu.add(Menu.NONE, 2, Menu.NONE, "Mis solicitudes")
		.setIcon(R.drawable.ic_format_list_bulleted_type);
		menu.add(Menu.NONE, 3, Menu.NONE, "Registrar incidencia")
		.setIcon(R.drawable.ic_bug);
		//menu.add(Menu.NONE, 4, Menu.NONE, "Nuevo desarrollo")
		//.setIcon(R.drawable.ic_code_braces);
		menu.add(Menu.NONE, 5, Menu.NONE, "Enviar sugerencias")
		.setIcon(R.drawable.ic_email_variant);
		menu.add(Menu.NONE, 6, Menu.NONE, "Preguntas frecuentes")
		.setIcon(R.drawable.ic_comment_question_outline);
		menu.add(Menu.NONE, 7, Menu.NONE, "Manual de usuario")
		.setIcon(R.drawable.ic_book_open_page_variant);
		return true;
	}



	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		// Se obtiene el rol del usuario conectado para comprobar si tiene permisos
		SharedPreferences sp = PreferenceManager
				.getDefaultSharedPreferences(SolicitudDesarrollo.this);
		//String nombreRol = sp.getString("nombreRol", null);
		String apiKeyUsuario = sp.getString("apiKeyUsuario", null);
		String rutaFicheroManualUsuario = sp.getString("rutaFicheroManualUsuario", null);

		switch (item.getItemId()) {
		case 1:
			Intent i = new Intent(this, MenuPrincipal.class);
			finish();
			startActivity(i);
			return true;
		case 2:
			Intent j = new Intent(this, MisSolicitudes.class);
			finish();
			startActivity(j);
			return true;
		case 3:
			Intent k = new Intent(this, RegistrarIncidencia.class);
			finish();
			startActivity(k);
			return true;
			//case 4:
			//	if (!nombreRol.equals("Cliente")) {			
			//		Intent l = new Intent(this, SolicitudDesarrollo.class);
			//		startActivity(l);							
			//	} else {
			//		Toast.makeText(SolicitudDesarrollo.this, "Su usuario no dispone de permisos para la funcionalidad solicitada",
			//				Toast.LENGTH_LONG).show();
			//		Intent in = new Intent(getApplicationContext(),
			//				MenuPrincipal.class);
			//		startActivity(in);			
			//	}
			//	return true;
		case 5:
			Intent m = new Intent(this, EnviarSugerencias.class);
			finish();
			startActivity(m);
			return true;
		case 6:
			Intent n = new Intent(this, PreguntasFrecuentes.class);
			finish();
			startActivity(n);
			return true;
		case 7:
			Uri uri = Uri.parse(rutaFicheroManualUsuario + "?key=" + apiKeyUsuario);
			Intent o = new Intent(Intent.ACTION_VIEW, uri);
			startActivity(o);
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}



	class RegistrandoSolicitudDesarrollo extends AsyncTask<String, String, String> {

		@Override
		protected void onPreExecute() {
			super.onPreExecute();
			pDialog = new ProgressDialog(SolicitudDesarrollo.this);
			pDialog.setMessage("Registrando incidencia...");
			pDialog.setIndeterminate(false);
			pDialog.setCancelable(true);
			pDialog.show();
		}

		protected void onProgressUpdate(String... values) {
			if (values != null) {
				for (String value : values) {
					Toast.makeText(SolicitudDesarrollo.this, value,
							Toast.LENGTH_SHORT).show();
				}
			}
		}


		@Override
		protected String doInBackground(String... args) {	

			String tituloSolicitud = cajaTextoTituloSolicitud.getText().toString();
			String descripcionSolicitud = cajaTextoDescripcionSolicitud.getText().toString();			

			// Obtiene el id de empresa y la clave codificada de autorizacin
			// del usuario usuario conectado mediante las propiedades compartidas
			SharedPreferences sp = PreferenceManager
					.getDefaultSharedPreferences(SolicitudDesarrollo.this);
			String idEmpresa = sp.getString("idEmpresa", null);
			String autorizacion = sp.getString("autorizacionBasica", null);			

			try {
				// Construye los parmetros de la solicitud de desarrollo a registrar					
				JSONObject jsonEntradaSolicitudDesarrollo = new JSONObject();
				JSONObject jsonHijoEntradaSolicitudDesarrollo = new JSONObject();

				jsonHijoEntradaSolicitudDesarrollo.put(Constantes.TAG_PROYECTO_ID, idEmpresa);
				jsonHijoEntradaSolicitudDesarrollo.put(Constantes.TAG_TITULO, tituloSolicitud);
				// Prioridad urgente
				jsonHijoEntradaSolicitudDesarrollo.put(Constantes.TAG_PRIORIDAD_ID, 4);

				// Obtiene el id de las solicitudes de tipo "Desarrollo"			
				String idTipoSolicitud = new ObtenerDatos()
				.obtenerIdTipoSolicitud(Constantes.TIPO_DESARROLLO, SolicitudDesarrollo.this, autorizacion);

				// Construye el JSON de entrada para la creacin de la solicitud
				jsonHijoEntradaSolicitudDesarrollo.put("tracker_id", idTipoSolicitud);
				jsonHijoEntradaSolicitudDesarrollo.put("description", descripcionSolicitud);
				jsonEntradaSolicitudDesarrollo.put("issue", jsonHijoEntradaSolicitudDesarrollo);				

				Log.d("Solicitud!", "Comenzando...");

				// Ejecuta el Post de la API REST para registrar la solicitud				
				JSONObject jsonRegistroSolicitudDesarrollo =
						jsonParser.ejecutarSolicitudHTTP
						(Constantes.SOLICITUDES_URL, "POST", jsonEntradaSolicitudDesarrollo, autorizacion, false);

				String respuestaServidor = jsonRegistroSolicitudDesarrollo.getString(Constantes.TAG_RESPUESTA);

				// Si se crea correctamente
				if (respuestaServidor.equals("201")){
					Log.d("Solicitud de Desarrollo registrada correctamente!",
							jsonRegistroSolicitudDesarrollo.toString());			
					finish();

				} else {
					Log.d("Error al registrar la solicitud de desarrollo!", 
							jsonRegistroSolicitudDesarrollo.getString(Constantes.TAG_ERRORES));				
					return "Error al registrar la solicitud de desarrollo!";					
				}
			} catch (JSONException e) {
				e.printStackTrace();
			}
			return "Solicitud de desarrollo registrada correctamente!";
		}

		protected void onPostExecute(String file_url) {			
			pDialog.dismiss();
			if (file_url != null) {
				Toast.makeText(SolicitudDesarrollo.this, file_url, Toast.LENGTH_LONG).show();
			}
		}		
	}
}